import selenium.webdriver
import unittest



class SeleniumHQTest(unittest.TestCase):
    def setUp(self):
        self.driver = selenium.webdriver.Firefox()
        self.url = "http://seleniumhq.org"
    def test_header_links(self):
        self.driver.get(self.url)
        elements = self.driver.find_elements_by_css_selector("div#header ul>li")
        self.assertTrue(len(elements) > 0)
        expected_link_list = ["Projects", "Download", "Documentation", "Support", "About"]
        actual_link_list = [el.text for el in elements]
        self.assertEquals( sorted(expected_link_list), sorted(actual_link_list))

    def test_about_selenium_heading(self):
        self.driver.get(self.url)
        about_link = self.driver.find_element_by_css_selector("div#header ul>li#menu_about>a")
        about_link.click()
        heading = self.driver.find_element_by_css_selector("#mainContent>h2")
        self.assertEquals(heading.text, "About Selenium")

    def tearDown(self):
        if self.driver:
            self.driver.quit()



# -*- coding: utf-8 -*-
import selenium.webdriver
import unittest



class PythonOrgTest(unittest.TestCase):
    def setUp(self):
        self.driver = selenium.webdriver.Firefox()

    def test_links(self):
        self.driver.get("http://www.python.org")
        elements = self.driver.find_elements_by_css_selector("ul.level-one li>a")
        assert len(elements) > 0
        link_list = [u"ABOUT", u"NEWS", u"DOCUMENTATION", u"DOWNLOAD", u"下载", u"COMMUNITY", u"FOUNDATION", u"CORE DEVELOPMENT"]
        for element in zip(elements, link_list):
            assert element[0].text == element[1], element[0].text

    def test_about_python_heading(self):
        self.driver.get("http://www.python.org")
        elements = self.driver.find_elements_by_css_selector("ul.level-one li>a")
        about_link = [ k for k in elements if k.text == u"ABOUT"][0]
        about_link.click()
        h1_title = self.driver.find_element_by_css_selector("h1.title")
        assert h1_title.text == u"About Python"

    def tearDown(self):
        if self.driver:
            self.driver.quit()



# -*- coding: utf-8 -*-
from holmium.core import HolmiumTestCase, PageObject, PageElement, PageElementMap, PageElements, Locators


class PythonOrgPage(PageObject):
    side_bar_links = PageElementMap( Locators.CSS_SELECTOR
                                        , "ul.level-one li"
                                        , key = lambda element : element.find_element_by_tag_name("a").text
                                        , value = lambda element: element.find_element_by_tag_name("a") )

    header_text = PageElement(Locators.CSS_SELECTOR, "h1.title")


class PythonOrgTest(HolmiumTestCase):

    def setUp(self):
        self.page = PythonOrgPage(self.driver, "http://www.python.org")

    def test_links(self):
        self.page.go_home()
        assert len(self.page.side_bar_links) > 0
        link_list = [u"ABOUT", u"NEWS", u"DOCUMENTATION", u"DOWNLOAD", u"下载", u"COMMUNITY", u"FOUNDATION", u"CORE DEVELOPMENT"]
        assert self.page.side_bar_links.keys() == link_list

    def test_about_python_heading(self):
        self.page.go_home()
        self.page.side_bar_links[u"ABOUT"].click()
        assert self.page.header_text.text == u"About Python"




from selenium.common.exceptions import NoSuchElementException, TimeoutException
from selenium.webdriver.support.ui import WebDriverWait, Select
from selenium.webdriver.remote.webelement import WebElement
import selenium.webdriver.common.by
import holmium
import inspect
import weakref
import types
import collections
class Locators(selenium.webdriver.common.by.By):
    """
    proxy class to access locator types
    """
    pass


def enhanced (web_element):
    """
    incase a higher level abstraction for a WebElement is available
    we will use that in PageObjects. (e.g. a select element is converted into
    :class:`selenium.webdriver.support.ui.Select`)
    """
    abstraction_mapping = {'select': Select}
    if web_element.tag_name in abstraction_mapping.keys():
        return abstraction_mapping[web_element.tag_name](web_element)
    return web_element


class PageElementList(list):
    """
    proxy to a standard list which would be stored in
    a :class:`holmium.core.PageObject`.
    """

    def __init__(self, instance, *args, **kwargs):
        self.instance = weakref.ref(instance)
        list.__init__(self, *args, **kwargs)

    def __getitem__(self, index):
        return list.__getitem__(self, index).__get__(self.instance(),
                                                     self.instance().__class__)

class PageElementDict(dict):
    """
    proxy to a standard dict which would be stored in
    a :class:`holmium.core.PageObject`.
    """

    def __init__(self, instance, *args, **kwargs):
        self.instance = weakref.ref(instance)
        dict.__init__(self, *args, **kwargs)

    def __getitem__(self, key):
        return dict.__getitem__(self, key).__get__(self.instance(),
                                                   self.instance().__class__)


class PageObject(object):
    """
    Base class for all page objects to extend from.
    """

    def __init__(self, driver, url=None, iframe=None):
        self.driver = driver
        self.home = url
        def update_element(el):
            if issubclass(el.__class__, ElementGetter):
                el.driver = self.driver

        for el in inspect.getmembers(self.__class__):
            if issubclass(el[1].__class__, list):
                for item in el[1]:
                    update_element(item)
                self.__setattr__(el[0], PageElementList(self, el[1]))
            elif issubclass(el[1].__class__, dict):
                for item in el[1].values():
                    update_element(item)
                self.__setattr__(el[0], PageElementDict(self, el[1]))
            else:
                update_element(el[1])

        if url:
            self.driver.get(url)
        if iframe:
            self.driver.switch_to_frame(iframe)

    def go_home(self):
        """
        returns the page object to the url it was initialized with
        """
        self.driver.get(self.home)


class ElementGetter(object):
    """
    internal class to encapsulate the logic used by
    :class:`holmium.core.PageElement`
    & :class:`holmium.core.PageElements`
    """

    def __init__(self, locator_type, query_string, base_element=None, timeout=1):
        self.query_string = query_string
        self.type = locator_type
        self.timeout = timeout
        self.driver = None
        self.base_element = base_element
        holmium.core.log.debug("locator:%s, query_string:%s, timeout:%d" %
                              (locator_type, query_string, timeout))

    def get_element(self, method = None):
        if self.base_element:
            if isinstance(self.base_element, types.LambdaType):
                el = self.base_element()
                _meth = getattr(el, method.im_func.func_name)
            elif isinstance(self.base_element, PageElement):
                _meth = getattr(self.base_element.__get__(self, self.__class__), method.im_func.func_name)
            elif isinstance(self.base_element, WebElement):
                _meth = getattr(self.base_element, "find_element")
            else:
                holmium.core.log.error("unknown base_element type used: %s" % type(self.base_element))
        else:
            _meth = method
        holmium.core.log.debug("looking up locator:%s, query_string:%s, timeout:%d" %
                              (self.type, self.query_string, self.timeout))
        if self.timeout:
            try:
                WebDriverWait(self.driver, self.timeout).until(lambda _: _meth(self.type, self.query_string))
            except TimeoutException:
                holmium.core.log.debug("unable to find element %s after waiting for %d seconds" % (self.query_string, self.timeout))

        return _meth(self.type, self.query_string)

    def extract( self ):
        """
        Extracts the :class:`selenium.webdriver.remote.webelement.WebElement`
        """
        return self.get_element()

class PageElement(ElementGetter):
    """
    Utility class to get a :class:`selenium.webdriver.remote.webelement.WebElement`
    by querying via one of :class:`holmium.core.Locators`
    """

    def __get__(self, instance, owner):
        if not instance:
            return self
        try:
            return enhanced(self.get_element(self.driver.find_element))
        except NoSuchElementException:
            return None

class PageElements(ElementGetter):
    """
    Utility class to get a collection of :class:`selenium.webdriver.remote.webelement.WebElement`
    objects by querying via one of :class:`holmium.core.Locators`
    """

    def __getitem__(self, idx):
        return lambda: self.__get__(self, self.__class__)[idx]

    def __get__(self, instance, owner):
        if not instance:
            return self
        try:
            return [enhanced(el) for el in self.get_element(self.driver.find_elements)]
        except NoSuchElementException:
            return []


class PageElementMap(PageElements):
    """
    Used to create dynamic dictionaries based on an element locator specified by one of
    :class:`holmium.core.Locators`.

    The wrapped dictionary is an :class:`collections.OrderedDict` instance.

    :param lambda key: transform function for mapping a key to a WebElement in the collection
    :param lambda value: transform function for the value when accessed via the key.
    """
    def __init__(self, locator_type, query_string=None, base_element=None, timeout =1
            , key  = lambda el:el.text, value = lambda el:el):
        PageElements.__init__(self, locator_type, query_string, base_element, timeout)
        self.key_mapper = key
        self.value_mapper = value

    def __get__(self, instance, owner):
        if not instance:
            return self
        try:
            return collections.OrderedDict((self.key_mapper(el), self.value_mapper(enhanced(el))) for el in self.get_element(self.driver.find_elements))
        except NoSuchElementException:
            return {}


    def __getitem__(self, key):
        return lambda:self.__get__(self, self.__class__)[key]





import pages
import unittest
import selenium.webdriver
from holmium.core import PageElement, Locators
class TextSearchArticle(unittest.TestCase):
    def setUp(self):
        self.driver = selenium.webdriver.Firefox()
        self.page = pages.WikiPedia(self.driver, "http://wikipedia.org")

    def test_text_search_alllangs(self):
        for language in self.page.languages:
            self.page.go_home()
            self.page.languages[language].click()
            self.page.search("google")
            assert self.page.article_title.text == "Google", self.page.article_title.text

    def tearDown(self):
        self.driver.quit()



# -*- coding: utf-8 -*-
from holmium.core import HolmiumTestCase, PageObject, PageElement, Locators, PageElementMap


class SeleniumHQPage(PageObject):
    nav_links = PageElementMap( Locators.CSS_SELECTOR
                                        , "div#header ul>li"
                                        , key = lambda element : element.find_element_by_tag_name("a").text
                                        , value = lambda element: element.find_element_by_tag_name("a") )

    header_text = PageElement(Locators.CSS_SELECTOR, "#mainContent>h2")


class SeleniumHQTest(HolmiumTestCase):
    def setUp(self):
        self.page = SeleniumHQPage(self.driver, "http://seleniumhq.org")

    def test_header_links(self):
        self.assertTrue( len(self.page.nav_links) > 0 )
        self.assertEquals( sorted(["Projects", "Download", "Documentation", "Support", "About"])
                        ,  sorted(self.page.nav_links.keys() ) )

    def test_about_selenium_heading(self):
        self.page.nav_links["About"].click()
        self.assertEquals(self.page.header_text.text, "About Selenium")




# -*- coding: utf-8 -*-
#

import sys, os
sys.path.insert(0, os.path.abspath('../../'))
import holmium.core

extensions = ['sphinx.ext.autodoc', 'sphinx.ext.autosummary','sphinx.ext.doctest', 'sphinx.ext.intersphinx', 'sphinx.ext.todo', 'sphinx.ext.pngmath', 'sphinx.ext.ifconfig', 'sphinx.ext.viewcode']
templates_path = ['_templates']
source_suffix = '.rst'
master_doc = 'index'
project = u'holmium.core'
copyright = u'2013, Projectgoth Inc.'
version = release = holmium.core.__version__

exclude_patterns = []
#add_module_names = True
pygments_style = 'sphinx'

html_theme = 'pyramid'
#html_theme_options = {}
#html_theme_path = []
html_title = "holmium"
#html_short_title = None
#html_logo = None
#html_favicon = None
html_static_path = ['_static']
#html_last_updated_fmt = '%b %d, %Y'
#html_use_smartypants = True
html_sidebars = {
           '**': ['globaltoc.html', 'sourcelink.html', 'searchbox.html'],
              'using/windows': ['windowssidebar.html', 'searchbox.html'],
              }
#html_additional_pages = {}
#html_domain_indices = True
#html_use_index = True
#html_split_index = False
#html_show_sourcelink = True
#html_show_sphinx = True
#html_show_copyright = True
htmlhelp_basename = 'holmiumcoredoc'

latex_documents = [
  ('index', 'holmiumcore.tex', u'holmium.core Documentation',
   u'Ali-Akber Saifee', 'manual'),
]
man_pages = [
    ('index', 'holmiumcore', u'holmium.core Documentation',
     [u'Ali-Akber Saifee'], 1)
]

texinfo_documents = [
  ('index', 'holmiumcore', u'holmium.core Documentation',
   u'Ali-Akber Saifee', 'holmiumcore', 'One line description of project.',
   'Miscellaneous'),
]
autodoc_default_flags = ["undoc-members","show-inheritance"]
intersphinx_mapping = {'http://docs.python.org/': None, "http://selenium-python.readthedocs.org/en/latest":None}



from holmium.core import PageObject, PageElement, PageElements, Locators, PageElementMap


class GoogleMain(PageObject):
    search_box = PageElement( Locators.NAME, "q", timeout = 1)
    google_buttons = PageElementMap ( Locators.CLASS_NAME, "gbts" , timeout = 1 )
    search_results = PageElements( Locators.CSS_SELECTOR, "li.g", timeout = 1)

    def __init__(self, *args, **kwargs):
        PageObject.__init__(self, *args, **kwargs)

    def search ( self, query ):
        self.google_buttons["Search"].click() # self.google_buttons behaves just like a dictionary
        self.search_box.clear() # self.search_box is now evaluated directly to a WebElement
        self.search_box.send_keys(query)
        self.search_box.submit()





import os
import unittest
from selenium import webdriver
import inspect
import imp
import holmium


class HolmiumTestCase(unittest.TestCase):
    """
    """
    #:
    browser_mapping = {"firefox": webdriver.Firefox,
                       "chrome": webdriver.Chrome,
                       "ie": webdriver.Ie,
                       "remote": webdriver.Remote}

    #:
    capabilities = {"firefox": webdriver.DesiredCapabilities.FIREFOX,
                    "chrome": webdriver.DesiredCapabilities.CHROME,
                    "ie": webdriver.DesiredCapabilities.INTERNETEXPLORER}

    @classmethod
    def setUp(self):
        """
        """
        pass

    @classmethod
    def setUpClass(self):
        """
        """
        self.driver = None
        base_file = inspect.getfile(self)
        config_path = os.path.join(os.path.split(base_file)[0], "config.py")
        try:
            config = imp.load_source("config", config_path)
            self.config = config.config[os.environ.get("HO_ENV", "prod")]
        except IOError:
            holmium.core.log.error("config.py not found for TestClass %s at %s" %
                                           (self, config_path))

        driver = os.environ.get("HO_BROWSER", "firefox").lower()
        remote_url = os.environ.get("HO_REMOTE", "").lower()
        args = {}
        if remote_url:
            cap = self.capabilities[driver]
            args = {"command_executor": remote_url,
                     "desired_capabilities": cap}
            driver = "remote"
        self.driver = self.browser_mapping[driver](**args)

    @classmethod
    def tearDownClass(self):
        if self.driver:
            self.driver.quit()

    @classmethod
    def tearDown(self):
        """
        """
        if self.driver:
            self.driver.delete_all_cookies()
